@@ -1,20 +1,15 @@
 
 from dataclasses import dataclass as _dataclass
+from dataclasses import field as _field
 from typing import List as _List, Dict as _Dict
-from copy import deepcopy as _deepcopy
-import pathlib as _pathlib
-import os as _os
-import json as _json
+from datetime import date as _date
 
 from ._inputfiles import InputFiles
 from ._disease import Disease
 from ._variableset import VariableSets, VariableSet
 
 
-__all__ = ["Parameters", "get_repository_version"]
-
-_default_parameters_path = _os.path.join(_pathlib.Path.home(),
-                                         "GitHub", "MetaWardsData")
+__all__ = ["Parameters", "get_repository_version", "get_repository"]
 
 _default_folder_name = "parameters"
 
@@ -22,14 +17,50 @@ _default_folder_name = "parameters"
 _repositories = {}
 
 
+def get_repository(repository: str = None, error_on_missing=True):
+    """Return the full path to the passed MetaWardsData repository.
+       This will return the default repository if None is passed
+
+       Returns a tuple of the repository path and version information
+    """
+    import os
+    from pathlib import Path
+
+    if repository is None:
+        repository = os.getenv("METAWARDSDATA", None)
+
+        if repository is None:
+            repository = os.path.join(Path.home(),
+                                      "GitHub", "MetaWardsData")
+
+    repository = os.path.expanduser(os.path.expandvars(repository))
+    repository = str(Path(repository).absolute().resolve())
+
+    if not os.path.exists(repository) or not os.path.isdir(repository):
+        if error_on_missing:
+            raise FileNotFoundError(
+                f"Cannot find the MetaWardsData repository "
+                f"at {repository}. Please follow the instructions "
+                f"at https://metawards.org/model_data.html to download "
+                f"and install the model data.")
+        else:
+            return (None, None)
+
+    v = get_repository_version(repository)
+    return (repository, v)
+
+
 def generate_repository_version(repository):
     """Try to run the './version' script within the passed repository,
        to generate the required 'version.txt' file
     """
     import subprocess
-    script = _os.path.join(repository, "version")
-    print(f"Regenerating version information using {script}")
-    subprocess.run(script, cwd=repository)
+    import os
+
+    if repository is not None:
+        script = os.path.join(repository, "version")
+        subprocess.run(script, cwd=repository, stdout=subprocess.DEVNULL,
+                       stderr=subprocess.DEVNULL)
 
 
 def get_repository_version(repository: str):
@@ -38,7 +69,9 @@ def get_repository_version(repository: str):
        Parameters
        ----------
        repository: str
-         The full path to the repository whose version should be obtained
+         The full path to the repository whose version should be obtained.
+         If this is 'None' then the default repository will be used
+         ($METAWARDSDATA or $HOME/GitHub/MetaWardsData)
 
        Returns
        -------
@@ -47,14 +80,20 @@ def get_repository_version(repository: str):
     """
     global _repositories
 
+    if repository is None:
+        return None
+
     if repository in _repositories:
         return _repositories[repository]
 
-    filename = _os.path.join(repository, "version.txt")
+    import os
+
+    filename = os.path.join(repository, "version.txt")
 
     try:
         with open(filename) as FILE:
-            version = _json.load(FILE)
+            import json
+            version = json.load(FILE)
             _repositories[repository] = version
             return version
     except Exception:
@@ -66,16 +105,22 @@ def get_repository_version(repository: str):
         generate_repository_version(repository)
 
         with open(filename) as FILE:
-            version = _json.load(FILE)
+            import json
+            version = json.load(FILE)
+            version["filepath"] = repository
             _repositories[repository] = version
             return version
     except Exception:
-        print(f"Could not find the repository version info in {filename}."
-              f"Please make sure that you have run './version' in that "
-              f"repository to generate the version info.")
-        _repositories[repository] = {"repository": "unknown",
+        from .utils._console import Console
+        Console.error(f"""
+Could not find the repository version info in {filename}. Please make sure
+that you have run './version' in that repository to generate the version
+info.""")
+        _repositories[repository] = {"filepath": repository,
+                                     "repository": "unknown",
                                      "version": "unknown",
-                                     "branch": "unknown"}
+                                     "branch": "unknown",
+                                     "is_dirty": True}
         return _repositories[repository]
 
 
@@ -98,8 +143,7 @@ class Parameters:
     """
     #: The set of input files that define the model Network
     input_files: InputFiles = None
-    #: The name of the UV file
-    uv_filename: str = None
+
     #: The set of parameters that define the disease
     disease_params: Disease = None
 
@@ -109,33 +153,51 @@ class Parameters:
 
     #: The fraction of day considered "day" for work, e.g. 0.7 * 24 hours
     length_day: float = 0.7
-    #: The fraction of day considered "day" for play
-    plength_day: float = 0.5
-    #: The number of initial infections
-    initial_inf: int = 5
 
     static_play_at_home: float = 0.0
     dyn_play_at_home: float = 0.0
 
-    data_dist_cutoff: float = 10000000.0
+    #: The cutoff distance in km beyond which workers or players cannot move
     dyn_dist_cutoff: float = 10000000.0
 
     play_to_work: float = 0.0
     work_to_play: float = 0.0
 
-    local_vaccination_thresh: int = 4
-    global_detection_thresh: int = 4
-    daily_ward_vaccination_capacity: int = 5
-    neighbour_weight_threshold: float = 0.0
-
-    #: proportion of daily imports
+    #: proportion of daily imports if there are additional infections
     daily_imports: float = 0.0
 
+    #: The index of the seeding ward if there are daily imports
+    ward_seed_index: int = None
+
+    #: The number of initial infections if there are daily imports
+    initial_inf: int = 5
+
+    #: how to treat the * state (stage 0). This should be a string
+    #: describing the method. Currently "R", "E" and "disable" are
+    #: supported. Not needed if the mapping is specified explicitly
+    #: in the disease
+    stage_0: str = "R"
+
     #: Seasonality parameter
     UV: float = 0.0
 
+    #: Date when transmission should be at a maximum
+    UV_max: _date = None
+
+    #: The global scale_uv. This is combined with the population and
+    #: per-ward level scale_uvs to give a single value
+    scale_uv: float = 1.0
+
+    #: The global background force of infection (FOI). This is combined
+    #: with the per-ward level bg_foi to give a single value
+    bg_foi: float = 0.0
+
     #: User parameters
-    user_params: _Dict[str, float] = None
+    user_params: _Dict[str, float] = _field(default_factory=dict)
+
+    #: All of the VariableSet adjustments that have been applied
+    #: to these parameters
+    adjustments: _List[VariableSet] = None
 
     _name: str = None
     _version: str = None
@@ -148,35 +210,54 @@ class Parameters:
     _repository_branch: str = None
     _repository_dir: str = None
 
+    #: The parameters for demographic sub-networks. If this is None then
+    #: the parameters are the same as the overall parameters
+    _subparams = None
+
     def __str__(self):
-        return f"Parameters {self._name}\n" \
-               f"loaded from {self._filename}\n" \
-               f"version: {self._version}\n" \
-               f"author(s): {self._authors}\n" \
-               f"contact(s): {self._contacts}\n" \
-               f"references(s): {self._references}\n" \
-               f"repository: {self._repository}\n" \
-               f"repository_branch: {self._repository_branch}\n" \
-               f"repository_version: {self._repository_version}\n\n" \
-               f"length_day = {self.length_day}\n" \
-               f"plength_day = {self.plength_day}\n" \
-               f"initial_inf = {self.initial_inf}\n" \
-               f"static_play_at_home = {self.static_play_at_home}\n" \
-               f"dyn_play_at_home = {self.dyn_play_at_home}\n" \
-               f"data_dist_cutoff = {self.data_dist_cutoff}\n" \
-               f"dyn_dist_cutoff = {self.dyn_dist_cutoff}\n" \
-               f"play_to_work = {self.play_to_work}\n" \
-               f"work_to_play = {self.work_to_play}\n" \
-               f"local_vaccination_thresh = " \
-               f"{self.local_vaccination_thresh}\n" \
-               f"global_detection_thresh = {self.global_detection_thresh}\n" \
-               f"daily_ward_vaccination_capacity = " \
-               f"{self.daily_ward_vaccination_capacity}\n" \
-               f"neighbour_weight_threshold = " \
-               f"{self.neighbour_weight_threshold}\n" \
-               f"daily_imports = {self.daily_imports}\n" \
-               f"UV = {self.UV}\n" \
-               f"additional_seeds = {self.additional_seeds}\n\n"
+        parts = []
+
+        for key, value in [("Parameters", self._name),
+                           ("loaded_from", self._filename),
+                           ("repository", self._repository),
+                           ("repository_branch", self._repository_branch),
+                           ("repository_version", self._repository_version),
+                           ("length_day", self.length_day),
+                           ("initial_inf", self.initial_inf),
+                           ("static_play_at_home", self.static_play_at_home),
+                           ("dyn_play_at_home", self.dyn_play_at_home),
+                           ("dyn_dist_cutoff", self.dyn_play_at_home),
+                           ("play_to_work", self.play_to_work),
+                           ("work_to_play", self.work_to_play),
+                           ("daily_imports", self.daily_imports),
+                           ("UV", self.UV),
+                           ("UV_max", self.UV_max),
+                           ("scale_uv", self.scale_uv),
+                           ("bg_foi", self.bg_foi),
+                           ("stage_0", self.stage_0)]:
+            if value is not None:
+                parts.append(f"* {key}: {value}")
+
+        return "\n".join(parts)
+
+    @staticmethod
+    def default():
+        """Return the default set of parameters"""
+        try:
+            (repository, v) = Parameters.get_repository()
+            repository_dir = repository
+            repository = v["repository"]
+            repository_branch = v["branch"]
+            repository_version = v["version"]
+
+            return Parameters(_repository=repository,
+                              _repository_dir=repository_dir,
+                              _repository_branch=repository_branch,
+                              _repository_version=repository_version)
+        except Exception:
+            pass
+
+        return Parameters()
 
     @staticmethod
     def get_repository(repository: str = None):
@@ -197,28 +278,7 @@ class Parameters:
              A tuple of the location on disk of the repository,
              plus the version information (git ID etc)
         """
-        if repository is None:
-            repository = _os.getenv("METAWARDSDATA")
-            if repository is None:
-                repository = _default_parameters_path
-
-        from pathlib import Path
-        import os
-        repository = os.path.expanduser(os.path.expandvars(repository))
-        repository = Path(repository).absolute().resolve()
-
-        if not os.path.exists(repository):
-            raise FileNotFoundError(
-                    f"Cannot find the MetaWardsData repository "
-                    f"at {repository}")
-
-        if not os.path.isdir(repository):
-            raise FileNotFoundError(
-                    f"Expected {repository} to be a directory containing "
-                    f"the MetaWardsData repository. It isn't?")
-
-        v = get_repository_version(repository)
-        return (repository, v)
+        return get_repository(repository)
 
     @staticmethod
     def load(parameters: str = "march29",
@@ -260,8 +320,9 @@ class Parameters:
         repository_dir = None
 
         if filename is None:
+            import os
             (repository, v) = Parameters.get_repository(repository)
-            filename = _os.path.join(repository, folder, f"{parameters}.json")
+            filename = os.path.join(repository, folder, f"{parameters}.json")
             repository_dir = repository
             repository = v["repository"]
             repository_branch = v["branch"]
@@ -275,53 +336,83 @@ class Parameters:
                 data = json.load(FILE)
 
         except Exception as e:
-            print(f"Could not find the parameters file {json_file}")
-            print(f"Either it does not exist of was corrupted.")
-            print(f"Error was {e.__class__} {e}")
-            print(f"To download the parameters type the command:")
-            print(f"  git clone https://github.com/metawards/MetaWardsData")
-            print(f"and then re-run this function passing in the full")
-            print(f"path to where you downloaded this directory")
+            from .utils._console import Console
+            Console.error(f"""
+Could not find the parameters file {json_file}. Either it does not exist or
+was corrupted. Error was {e.__class__} {e}. "Please see
+https://metawards.org/model_data for instructions on how to download and
+set the model data.""")
             raise FileNotFoundError(f"Could not find or read {json_file}: "
                                     f"{e.__class__} {e}")
 
         par = Parameters(
-                length_day=data.get("length_day", 0.7),
-                plength_day=data.get("plength_day", 0.5),
-                initial_inf=data.get("initial_inf", 0),
-                static_play_at_home=data.get("static_play_at_home", 0.0),
-                dyn_play_at_home=data.get("dyn_play_at_home", 0.0),
-                data_dist_cutoff=data.get("data_dist_cutoff", 10000000.0),
-                dyn_dist_cutoff=data.get("dyn_dist_cutoff", 10000000.0),
-                play_to_work=data.get("play_to_work", 0.0),
-                work_to_play=data.get("work_to_play", 0.0),
-                local_vaccination_thresh=data.get(
-                                    "local_vaccination_threshold", 4),
-                global_detection_thresh=data.get(
-                                    "global_detection_threshold", 4),
-                daily_ward_vaccination_capacity=data.get(
-                                "daily_ward_vaccination_capacity", 5),
-                neighbour_weight_threshold=data.get(
-                                "neighbour_weight_threshold", 0.0),
-                daily_imports=data.get("daily_imports", 0),
-                UV=data.get("UV", 0.0),
-                _name=data.get("name", parameters),
-                _authors=data.get("author(s)", "unknown"),
-                _version=data.get("version", "unknown"),
-                _contacts=data.get("contact(s)", "unknown"),
-                _references=data.get("reference(s)", "none"),
-                _filename=json_file,
-                _repository=repository,
-                _repository_dir=repository_dir,
-                _repository_branch=repository_branch,
-                _repository_version=repository_version
-                )
-
-        print("Using parameters:")
-        print(par)
+            length_day=data.get("length_day", 0.7),
+            initial_inf=data.get("initial_inf", 0),
+            static_play_at_home=data.get("static_play_at_home", 0.0),
+            dyn_play_at_home=data.get("dyn_play_at_home", 0.0),
+            dyn_dist_cutoff=data.get("dyn_dist_cutoff", 10000000.0),
+            play_to_work=data.get("play_to_work", 0.0),
+            work_to_play=data.get("work_to_play", 0.0),
+            daily_imports=data.get("daily_imports", 0),
+            UV=data.get("UV", 0.0),
+            UV_max=data.get("UV_max", None),
+            scale_uv=data.get("scale_uv", 1.0),
+            bg_foi=data.get("bg_foi", 0.0),
+            _name=data.get("name", parameters),
+            _authors=data.get("author(s)", "unknown"),
+            _version=data.get("version", "unknown"),
+            _contacts=data.get("contact(s)", "unknown"),
+            _references=data.get("reference(s)", "none"),
+            _filename=json_file,
+            _repository=repository,
+            _repository_dir=repository_dir,
+            _repository_branch=repository_branch,
+            _repository_version=repository_version
+        )
 
         return par
 
+    def __getitem__(self, demographic: str):
+        """Return the parameters that should be used for the demographic
+           subnetwork called 'demographic'. If these have not been set
+           specifically then the parameters for the overall network
+           are used
+        """
+        if demographic == "overall":
+            return self
+
+        if self._subparams is None:
+            self._subparams = {}
+
+        if demographic not in self._subparams:
+            from copy import deepcopy
+            self._subparams[demographic] = deepcopy(self)
+            self._subparams[demographic]._subparams = {}
+
+        return self._subparams[demographic]
+
+    def copy(self, include_subparams: bool = False):
+        """Return a safe copy of these parameters, which does not
+           include any subnetwork parameters if 'include_subparams' is False
+        """
+        from copy import deepcopy
+        params = deepcopy(self)
+
+        if not include_subparams:
+            params._subparams = None
+
+        return params
+
+    def specialised_demographics(self) -> _List[str]:
+        """Return the names of demographics that have specialised
+           parameters that are different to those of the overall
+           network
+        """
+        if self._subparams is None:
+            return []
+        else:
+            return list(self._subparams.keys())
+
     def add_seeds(self, filename: str):
         """Add an 'additional seeds' file that can be used to
            seed wards with new infections at different times and
@@ -336,15 +427,14 @@ class Parameters:
         if self.additional_seeds is None:
             self.additional_seeds = []
 
-        if not _os.path.exists(filename):
-            f = _os.path.join(self._repository_dir, "extra_seeds", filename)
+        import os
 
-            if _os.path.exists(f):
-                filename = f
-            else:
-                raise FileExistsError(
-                        f"Unable to find extra seeds file {filename} in "
-                        f"the current directory or in {f}")
+        if not os.path.exists(filename):
+            if self._repository_dir is not None:
+                f = os.path.join(self._repository_dir, "extra_seeds", filename)
+
+                if os.path.exists(f):
+                    filename = f
 
         self.additional_seeds.append(filename)
 
@@ -359,30 +449,32 @@ class Parameters:
              If a string is passed then the InputFiles will be loaded
              based on that string.
         """
-        if isinstance(input_files, str):
-            input_files = InputFiles.load(input_files,
-                                          repository=self._repository_dir)
+        if isinstance(input_files, InputFiles):
+            from copy import deepcopy
+            self.input_files = deepcopy(input_files)
+            return
 
-        print("Using input files:")
-        print(input_files)
+        self.input_files = InputFiles.load(input_files,
+                                           repository=self._repository_dir)
 
-        self.input_files = _deepcopy(input_files)
-
-    def set_disease(self, disease: Disease):
+    def set_disease(self, disease: Disease, silent: bool = True):
         """"Set the disease that will be modelled
 
             Parameters:
               disease: The disease to be modelled. If a string is passed
               then the disease will be loaded using that string
+              silent: Whether or not to suppress printing out the disease
         """
         if isinstance(disease, str):
             disease = Disease.load(disease,
                                    repository=self._repository_dir)
 
-        print("Using disease")
-        print(disease)
+        if not silent:
+            from .utils._console import Console
+            Console.print(disease, markdown=True)
 
-        self.disease_params = _deepcopy(disease)
+        from copy import deepcopy
+        self.disease_params = deepcopy(disease)
 
     def set_variables(self, variables: VariableSet):
         """This function sets the adjustable variable values to those
@@ -400,7 +492,8 @@ class Parameters:
            params: Parameters
              A copy of this set of parameters with the variables adjusted
         """
-        params = _deepcopy(self)
+        from copy import deepcopy
+        params = deepcopy(self)
 
         if isinstance(variables, dict):
             variables = VariableSet(variables)
